import requests
import logging

__version__ = '0.3.2'

class Client:
    def __init__(self, p2_store_endpoint, timeout:int = 10):
        self.log = logging.getLogger(__name__)
        self.timeout = timeout
        self.p2_store_endpoint = p2_store_endpoint

    def get_campaigns(self):
        url = self.p2_store_endpoint+'/campaigns'
        try:
            resp = requests.get( url, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting all campaigns")
            return None

    def get_campaign_of_id(self, id:int):
        url = self.p2_store_endpoint + '/campaign/' + str(id)
        try:
            resp = requests.get( url, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error(f"Error getting campaign {id}")
            return None
        pass

    def get_experiments(self, *args, **kargs):
        url = self.p2_store_endpoint+'/experiments'
        if len(args):
            kargs = { **args[0], **kargs }
        try:
            resp = requests.get(url, params=kargs, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting all experiments")
            return None

    def get_experiments_as_dict(self):
        url = self.p2_store_endpoint+'/experiments_as_dict'
        try:
            resp = requests.get( url, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting all experiments")
            return None

    def get_experiment(self, filepath):
        url = self.p2_store_endpoint+'/experiment/'+filepath
        try:
            resp = requests.get( url, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting experiment {}: {}".format(filepath, e))
            return None

    def add_experiment(self, filename, data):
        url = self.p2_store_endpoint+'/experiment/'+filename
        try:
            resp = requests.post(
                url,
                json=data,
                timeout=self.timeout)
            return resp.json() if (resp.status_code == 201) else None
        except Exception as e:
            self.log.error("Error creating experiment {}".format(e))
            return None

    def delete_experiment(self, filename):
        url = self.p2_store_endpoint+'/experiment/'+filename
        resp = requests.delete( url, timeout=self.timeout)
        return resp.json() if (resp.status_code == 200) else None

    def create_session(self, exp_id, session):
        url = self.p2_store_endpoint+'/exp_session/'+exp_id
        try:
            resp = requests.post(
                url, 
                json=session,
                timeout=self.timeout)
            return exp_id if (resp.status_code == 201) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error creating session: {}".format(e))
            raise e

    def get_version(self):
        try:
            res = requests.get(self.p2_store_endpoint+'/version', timeout=self.timeout)
            if (res.status_code != 200):
                self.log.warning("No version found")
                return None
            else:
                return res.json()
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting p2store version {}".format(e))
            return None

    def get_sessions(self, **filter):
        try:
            res = requests.get(self.p2_store_endpoint+'/exp_session', 
                params=filter,
                timeout=self.timeout
            )
            if (res.status_code != 200):
                self.log.warning("No session found")
                return None
            else:
                sessions = res.json()
                if (len(sessions) == 0):
                    self.log.warning("No session found")
                return sessions
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting sessions: {}".format(e))
            return None

    def get_exp_log(self, exp_id, log_type):
        try:
            url = self.p2_store_endpoint+'/exp_log/'+exp_id+'/'+ str(log_type)
            res = requests.get(url, timeout=self.timeout)
            return res.json()
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting session {} log: {}".format(exp_id, e))
            return None

    def update_session( self, exp_id, attrs ):
        try:
            res = requests.put(self.p2_store_endpoint+"/exp_session/{}".format(exp_id), json=attrs)
            return exp_id if res.status_code == 200 else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error updating session {}: {}".format(exp_id, e))
            return None

    def update_log( self, exp_id, log ):
        try: 
            res = requests.post(self.p2_store_endpoint+"/exp_log/{}".format(exp_id), json=log)
            return exp_id if res.status_code == 201 else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error updating log {}: {}".format(exp_id, e))
            return None

    def execute_command ( self, cmd, attrs ):
        try:
            res = requests.post(
                    self.p2_store_endpoint+"/cmd", 
                    json= {'cmd': cmd, 'attrs':attrs},
                    )
            return res.json() if res.status_code == 200 else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error posting cmd {}: {}".format(cmd, e))
            return None

    def get_config(self, component):
        url = self.p2_store_endpoint+'/config/'+component
        try:
            resp = requests.get( url, timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting config for component {}: {}".format(component, e))
            return None

    def get_k8s_resources(self, *args, **kargs):
        if len(args):
            kargs = { **args[0], **kargs }
        url = self.p2_store_endpoint+'/k8s_resources'
        try:
            resp = requests.get( 
                url, 
                params=kargs,
                timeout=self.timeout)
            return resp.json() if (resp.status_code == 200) else None
        except requests.exceptions.RequestException as e:
            self.log.error("Error getting resources matching filter: {}".format(filter, e))
 
