import unittest
import os
import sys
from psycopg2 import connect, extensions, sql
from datetime import datetime

from p2.store.client import Client
from p2.tools.prepare_db import setUpDB, tearDownDB


class TestClient(unittest.TestCase):

    def setUp(self):
        self.p2_store_url = os.environ.get('P2_STORE_URL')

    def test_get_version(self):
        client = Client(self.p2_store_url, timeout=60)
        version = client.get_version()
        self.assertIsNotNone(version)

    def test_get_campaigns(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        self.assertEqual(len(campaigns), 2)

    def test_get_campaign(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        campaign = client.get_campaign_of_id(campaigns[0]['id'])
        self.assertEqual(campaign['id'], campaigns[0]['id'])

    def test_get_experiment(self):
        client = Client(self.p2_store_url)
        data = client.get_experiment('exp_1')
        self.assertEqual(data, {"title":"experiment 1"})

    def test_get_experiments(self):
        client = Client(self.p2_store_url)
        records = client.get_experiments(filename='exp_1')
        self.assertEqual(len(records), 1)
        self.assertEqual(records[0]['filename'], 'exp_1')

    def test_get_experiments_of_campaign(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        records = client.get_experiments(
                campaigns[0]['experiments']
                )
        self.assertEqual(len(records), 5)
        self.assertEqual(records[0]['filename'], 'exp_1')

    def test_get_experiments_as_dict(self):
        client = Client(self.p2_store_url)
        records = client.get_experiments_as_dict()
        self.assertEqual(len(records.keys()), 5)

    def test_add_delete_experiment(self):
        client = Client(self.p2_store_url)
        exp = client.add_experiment(
                'exp_6', {'k1':'v1'}
        )
        self.assertIsNotNone(exp)
        self.assertEqual(exp['filename'], 'exp_6')

        exp = client.delete_experiment('exp_6')
        self.assertEqual(exp, 'exp_6')

    def test_create_session(self):
        client = Client(self.p2_store_url)
        time = datetime.now()
        campaigns = client.get_campaigns()
        campaign_id = campaigns[0]['id']
        id = client.create_session('new-session', {
            'filepath': 'exp_1',
            'campaign_id': campaign_id,
            'schedule': '0 0 * * *',
            'start_time': str(time),
            'end_time': str(time),
            'status': 'completed'
        })
        self.assertEqual(id, 'new-session')

    def test_get_sessions(self):
        client = Client(self.p2_store_url)
        sessions = client.get_sessions(exp_id='1')
        self.assertEqual(len(sessions), 1)
        self.assertEqual(sessions[0]['exp_id'], '1')

    def test_update_session(self):
        client = Client(self.p2_store_url)
        id = client.update_session('1', {'status': 'scheduled'})
        self.assertEqual(id, '1')
        sessions = client.get_sessions(exp_id='1')
        self.assertEqual(sessions[0]['status'], 'scheduled')

    def test_update_log(self):
        client = Client(self.p2_store_url)
        id = client.update_log(
                '1', {
                    'log_type': 1,
                    'log_text': 'log text'
                })
        self.assertEqual(id, '1')

    def test_get_exp_log(self):
        client = Client(self.p2_store_url)
        log = client.get_exp_log('1', 1)
        self.assertEqual(log[0]['log_type'], 1)
        self.assertEqual(log[0]['log_text'], 'log text type 1')

    def test_execute_command(self):
        client = Client(self.p2_store_url)
        session = client.execute_command(
                cmd="select * from exp_session where exp_id = %s",
                attrs=['1']
        )
        self.assertEqual(session[0]['status'], 'completed')

    def test_get_config(self):
        client = Client(self.p2_store_url)
        config = client.get_config('p2.scheduler')
        self.assertEqual(config['spare_days_per_month'], 1)

    def test_get_k8s_resources(self):
        client = Client(self.p2_store_url)
        resources = client.get_k8s_resources(
                context='minikube',
                kind='namespace'
                )
        self.assertEqual(len(resources),3)
        self.assertEqual(resources[0]['id'], 1)

    def test_get_k8s_resources_of_campaign(self):
        client = Client(self.p2_store_url)
        campaigns = client.get_campaigns()
        resources = client.get_k8s_resources(
                campaigns[0]['resources']
                )
        self.assertEqual(len(resources),1)
        self.assertEqual(resources[0]['id'], 1)
